#include <stdlib.h>
#include <stdio.h>
#include <errno.h>
#include <limits.h>
#include <unistd.h>
#include <string.h>

int
main(int argc, char **argv)
{
	FILE *ofile = NULL;
	int c;
	size_t n;
	int status = 0;
	int close_stdout = 0;
	
	struct timeout_map {
		size_t off;
		unsigned long delay;
	};
#define TMAX 128
	static struct timeout_map tmap[TMAX];
	static int tmax = 0;
	int tind = 0;
#define USEC_IN_SEC (1e6)

	while ((c = getopt(argc, argv, "ce:f:t:")) != EOF) {
		switch (c) {
		case 'c':
			close_stdout = 1;
			break;
			
		case 'e': {
			char *p;
			unsigned long n;

			n = strtoul(optarg, &p, 10);
				
			if ((n == 0 && errno == ERANGE) || n > 127) {
				fprintf(stderr, "invalid exit status\n");
				return 2;
			}
			status = n;
			break;
		}
			
		case 'f':
			if (strcmp(optarg, "-") == 0)
				ofile = stdout;
			else if ((ofile = fopen(optarg, "w")) == NULL) {
				perror(optarg);
				return 1;
			}
			break;

		case 't':
			while (*optarg) {
				char *p;
				unsigned long n;

				if (tmax == TMAX) {
					fprintf(stderr, "timeout map overflow\n");
					return 2;
				}
				
				n = strtoul(optarg, &p, 10);
				
				if (n == 0 && errno == ERANGE) {
					fprintf(stderr, "invalid offset (near %s)\n",
						optarg);
					return 2;
				}
				tmap[tmax].off = n;

				optarg = p;
				if (*optarg == 0)
					break;
				if (*optarg != '=') {
					fprintf(stderr, "expected '=' (near %s)\n",
						optarg);
					return 2;
				}
				optarg++;

				n = strtoul(optarg, &p, 10);
				if (n == 0 && errno == ERANGE) {
					fprintf(stderr, "invalid delay (near %s)\n",
						optarg);
					return 2;
				}
				if (*p == 's') {
					if (UINT_MAX / USEC_IN_SEC < n) {
						fprintf(stderr, "delay too big (near %s)\n",
							optarg);
						return 2;
					}
					n *= USEC_IN_SEC;
					p++;
				}
				tmap[tmax].delay = n;
				tmax++;

				optarg = p;
				if (*optarg == ':')
					optarg++;
			}
			break;	
			
		default:
			return 2;
		}
	}

	if (!ofile && (ofile = fopen("/dev/null", "w")) == NULL) {
		perror("/dev/null");
		return 1;
	}	

	if (close_stdout) {
		if (ofile == stdout) {
			fprintf(stderr, "%s: '-c' and '-f -' used together\n",
				argv[0]);
			return 2;
		}
		fclose(stdout);
		if ((stdout = fopen("/dev/null", "w")) == NULL) {
			perror("/dev/null");
			return 1;
		}
		fclose(stderr);
		stderr = stdout;
	}
	
	for (n = 0; (c = getchar()) != EOF; n++) {
		if (tind < tmax && tmap[tind].off == n) {
			usleep(tmap[tind].delay);
			tind++;
		}
		fputc(c, ofile);
	}
	
	if (tind < tmax)
		usleep(tmap[tind].delay);

	fclose(ofile);
	    
	return status;
}

	
